{$mode objfpc}
unit u_branch;

interface
uses u_vector;

type
   PInt_Arr = array of Integer;
   TBranchUpdate = procedure (Count : Integer;
      Left, Height, Width, Speed : Integer);

type
   TBranch =
   record
      // Private
      Height : Integer;
      Center_X, Width : Integer;
      Count, Mass : Integer;

      // Public
      Arr : PInt_Arr;
   end;

   // Constructor and Destructor
   function Branch_Create (ACenter, AHeight, AWidth : Integer) : TBranch;
   procedure Branch_Destroy (var B : TBranch);

   // Methods
   procedure Show (B : TBranch);
   procedure Update (var B : TBranch; X, Add_Mass : Integer;
                     Proc : TBranchUpdate);

   // Public properties
   function Get_Left (B : TBranch) : Integer;
   function Get_Right (B : TBranch) : Integer;
   function Get_Height (B : TBranch) : Integer;
   function Get_Arr (B : TBranch; Index : Integer) : Integer;

type
   Branch_Vector = specialize Vector<TBranch>;
var
   Branches : Branch_Vector;


implementation
uses u_graph;

   procedure Set_Arr (B : TBranch; Index : Integer; Value : Integer);
   forward;


   function Branch_Create (ACenter, AHeight, AWidth : Integer) : TBranch;
   var i : Integer;
   begin
      with Result do
      begin
         Center_X := ACenter;
         Height := AHeight;
         Width := AWidth;
         SetLength(Arr, AWidth);
         Mass := 3 * AWidth;
         Count := 0;

         for i := 0 to AWidth - 1 do
         begin
	    Arr[i] := 1;
	 end;
      end;
   end; { Branch_Create }

   procedure Branch_Destroy (var B : TBranch);
   begin
     B.Arr := nil;
   end; { Branch_Destroy }

   function Get_Arr (B : TBranch; Index : Integer) : Integer;
   begin
      Result := B.Arr[Index];
   end; { Get_Arr }

   function Get_Height (B : TBranch) : Integer;
   begin
      Result := B.Height;
   end; { Get_Height }

   function Get_Left (B : TBranch) : Integer;
   begin
      Result := B.Center_X - (B.Width div 2);
   end; { Get_Left }

   function Get_Right (B : TBranch) : Integer;
   begin
      Result := B.Center_X + (B.Width div 2);
   end; { Get_Right }

const
   Branch_Linear = 4;

   procedure go_linear (B : TBranch; X : Integer);
   var
      curr, i : Integer;
      value : Integer;
      save : array[-Branch_Linear .. Branch_Linear] of Integer;
      Sum : Double;
   begin
      for curr := X - Branch_Linear to X + Branch_Linear do
      begin
         Sum := 0.0;
         for i := curr - Branch_Linear to curr + Branch_Linear do
         begin

            if i < 0 then
               value := 0
            else if i >= B.Width then
               value := 0
            else
               value := Get_Arr (B, i);

            Sum := Sum + value;
         end;

         Sum := Sum / (2 * Branch_Linear + 1);
         save[curr - X] := Trunc (Sum);
      end;

      Set_Xor_Mode;
      for curr := X - Branch_Linear to X + Branch_Linear do
      begin
         if (curr > 0) and (curr < B.Width) then
         begin
            Draw_Line (Get_Left (B) + curr, Get_Height (B),
                       Get_Left (B) + curr, Get_Height (B) + Get_Arr (B, curr));

            Draw_Line (Get_Left (B) + curr, Get_Height (B),
                       Get_Left (B) + curr, Get_Height (B) - Get_Arr (B, curr));

            Draw_Line (Get_Left (B) + curr, Get_Height (B),
                       Get_Left (B) + curr, Get_Height (B) + save[curr - X]);

            Draw_Line (Get_Left (B) + curr, Get_Height (B),
                       Get_Left (B) + curr, Get_Height (B) - save[curr - X]);
            Set_Arr (B, curr, save[curr - X]);
         end;
      end;
      Set_Copy_Mode;
   end; { go_linear }

   procedure Set_Arr (B : TBranch; Index : Integer; Value : Integer);
   begin
      B.Arr[Index] := Value;
   end; { Set_Arr }

   procedure Show (B : TBranch);
   begin
      // nothing
   end; { Show }

   procedure Update (var B : TBranch; X, Add_Mass : Integer;
                     Proc : TBranchUpdate);
   var
      i, px : Integer;
   begin
      px := X - Get_Left (B);
      if (px >= 0) and (px < B.Width) then
      begin
         Set_Arr (B, px, Get_Arr (B, px) + 2);
         B.Count := B.Count + 1;

         B.Mass := B.Mass - Add_Mass;
         if B.Mass > 0 then
            go_linear (B, px)
         else
         begin
            FillWord (B.Arr, Length (B.Arr), 0);
            for i := 0 to B.Width - 1 do
            begin
               go_linear (B, i);
            end;

            Show (B);

            Proc (B.Count div 3, Get_Left (B), Get_Height (B), B.Width, 17);
            B.Mass := 3 * B.Width;
         end;
      end;
   end; { Update }

end.
