{$mode objfpc}
unit u_equalizer;

interface

const
   Linear = 4;

   Start = -2 * Linear;
   Finish = 800 + 2 * Linear;

type
   EqArr = array[Start .. Finish] of Integer;
   TEqualizer =
   record
      Arr : EqArr;
   end;

   // Constructor
   procedure Equalizer_Create (var E : TEqualizer);

   // Methods
   procedure Decrease_Level (var E : TEqualizer);
   function Get_Value (E : TEqualizer; Index : Integer;
                       var Range_Out : Boolean) : Integer;
   procedure Update (var E : TEqualizer; px : Integer);

var
   Ground : TEqualizer;

implementation
uses u_graph;

   procedure go_linear (var E : TEqualizer; x : Integer);
   forward;

   procedure Decrease_Level (var E : TEqualizer);
   var i : Integer;
   begin
      for i := Start to Finish do
      begin
         if E.Arr[i] < Get_ScrHeight - 5 then
         begin
            E.Arr [i] := E.Arr [i] + 5;
         end;
      end;
   end; { Decrease_Level }

   // constructor
   procedure Equalizer_Create (var E : TEqualizer);
   var i : Integer;
   begin
      Set_Color (White_Color);
      for i := Start to Finish do
      begin
         E.Arr[i] := Get_ScrHeight - Random (10);
      end;

      for i := 0 to 800 do
      begin
         go_linear (E, i);
      end;
      // Restore colors
   end; { Equalizer_Create }

   function Get_Value (E : TEqualizer; Index : Integer;
                       var Range_Out : Boolean) : Integer;
   begin
      Range_Out := (Index < -2 * Linear) or (Index > Get_ScrWidth + 2 * Linear);
      if Range_Out then
         Result := -1
      else
         Result := E.Arr[Index];
   end; { Get_Value }

   procedure go_linear (var E : TEqualizer; x : Integer);
   var
      s : Double;
      save : array[-Linear .. Linear] of Integer;
      i, curr : Integer;
   begin
      for curr := x - Linear to x + Linear do
      begin
         s := 0.0;
         for i := curr - Linear to curr + Linear do
         begin
            s := s + E.Arr[i];
         end;
         s := s / (2 * Linear + 1);
         save[curr - x] := Trunc (s);
      end;

      for curr := x - Linear to x + Linear do
      begin
         if E.Arr[curr] > save[curr - x] then
         begin
            if (curr >= 0) and (curr < Get_ScrWidth) then
            begin
               Set_Color (White_Color);
               Draw_Line (curr, E.Arr[curr], curr, save[curr - x]);
               // Restore colors
            end;
         end
         else
         begin
            if (curr >= 0) and (curr < Get_ScrWidth) then
            begin
               Set_Color (Black_Color);
               Draw_Line (curr, E.Arr[curr], curr, save[curr - x]);
               // Restore colors
            end;
         end;
         E.Arr[curr] := save[curr - x];
      end;

   end; { go_linear }

   procedure Update (var E : TEqualizer; px : Integer);
   begin
      E.Arr[px] := E.Arr[px] - 1;
      if (px >= 0) and (px < Get_ScrWidth) then
      begin
         go_linear (E, px);
      end;
   end; { Update }

end.
