{$mode objfpc}

unit u_fern;

interface

   procedure Draw_Fern (Level : Integer; Px, Py : Integer; L, U : Double;
                        has_branches : Boolean = False);


implementation
uses u_graph, u_random, u_branch, u_light;

const
   Pi_Div_30 = Pi / 30.0;
   Pi_Div_30_Mul_14 = 14.0 * Pi_Div_30;


const
   // ColorizeLevel = 8;
   LightsOnLevel = 14;

   function check_line (xst, yst, xfn, yfn : Integer;
                        var len : Integer) : Boolean;
   var
      Sqr_X : Double;
      Sqr_Y : Double;
   begin
      Sqr_X := Sqr (xfn - xst);
      Sqr_Y := Sqr (yfn - yst);
      len := Trunc (Sqrt (Sqr_X + Sqr_Y));
      Result := (len > 10) and (Sqr_X > Sqr_Y);
   end; { check_line }


   procedure Line_To (Level : Integer; X, Y : Integer; L, U : Double;
                      has_branches : Boolean);
   var
      i, len : Integer;
      found : Boolean;
      pnt_x, pnt_y : Integer;

   begin
      pnt_x := (X + Trunc (L * Cos (U)));
      pnt_y := (Y - Trunc (L * Sin (U)));
      if has_branches then
      begin

         if check_line (X, Y, pnt_x, pnt_y, len) then
         begin
            found := false;
            for i := 0 to Branches.Count - 1 do
            begin

               found := Get_Height (Branches.Element (i)) = ((Y + pnt_y) div 2);
               if found then Break;

            end;

            if not found then
            begin
               Branches.Append (Branch_Create ((X + pnt_x) div 2, (Y + pnt_y) div 2, len));
            end;
         end;
      end;

      if has_branches then
      begin
         if Get_Prob (97) then
            Set_Color (DkGreen_Color)
         else
            Set_Color (LtGreen_Color);
      end
      else
      begin
         Set_Color (LtGray_Color);
      end;

      Draw_Line (X, Y, pnt_x, pnt_y);

      if Level = LightsOnLevel then
      begin
         if has_branches and Get_Prob (75) then
         begin
            LightList.Append (Light_Create (pnt_x, pnt_y,
              Light_Colors[Random (maxLightColors)]));
         end;
      end;

   end; { Line_To }


const
   min = 1.0;

   procedure Draw_Fern (Level : Integer; Px, Py : Integer; L, U : Double;
                        has_branches : Boolean);
   var x, y : Integer;
   begin
      if L > min then
      begin

         if (Level > 1) and (Level < 5) then
         begin
            Draw_Fern (5, Px, Py, 21.0, U, has_branches);
         end;

         Line_To (Level, Px, Py, L, U, has_branches);

         x := (Px + Trunc (L * Cos (U)));
         y := (Py - Trunc (L * Sin (U)));

         Draw_Fern (Level + 1,
              x, y, L * 0.45, U - Pi_Div_30_Mul_14, has_branches);
         Draw_Fern (Level + 1,
              x, y, L * 0.45, U + Pi_Div_30_Mul_14, has_branches);
         Draw_Fern (Level + 1,
              x, y, L * 0.73, U + Pi_Div_30, has_branches);

      end;
   end; { Draw_Fern }

end.
