{$mode objfpc}
unit u_graph;

interface
uses Graph;

type
  Color_Type = Integer;

const
   // Color Constants
   Black_Color {: Color_Type} = Black;
   White_Color {: Color_Type} = White;

   DkGreen_Color {: Color_Type} = Green;
   LtGreen_Color {: Color_Type} = LightGreen;
   LtGray_Color {: Color_Type} = LightGray;

   DkBlue_Color {: Color_Type} = Blue;
   LtBlue_Color {: Color_Type} = LightBlue;
   DkRed_Color {: Color_Type} = Red;
   LtRed_Color {: Color_Type} = LightRed;
   DkYellow_Color {: Color_Type} = Brown;
   LtYellow_Color {: Color_Type} = Yellow;
   DkMagenta_Color {: Color_Type} = Magenta;
   LtMagenta_Color {: Color_Type} = LightMagenta;

procedure OpenGraphix;
procedure CloseGraphix;

function Get_ScrWidth : Integer;
function Get_ScrHeight : Integer;

procedure Draw_Circle (X, Y : Integer; R : Integer);
procedure Draw_Line (X_Start, Y_Start, X_Finish, Y_Finish : Integer);
procedure Fill_Circle (X, Y : Integer; R : Integer);

procedure Set_Pixel (X, Y : Integer; Color : Color_Type);
function Get_Pixel (X, Y : Integer) : Color_Type;

procedure Set_Xor_Mode;
procedure Set_Copy_Mode;
procedure Set_Color (Color : Color_Type);
procedure Set_SolidBrush (Color : Color_Type);

implementation

var
   grDriver, grMode, ErrCode : SmallInt;
   
procedure OpenGraphix;
begin
   grDriver := d8bit; grMode := m800x600;
   InitGraph (grDriver, grMode, '');
   ErrCode := GraphResult;
   if ErrCode <> grOk then
   begin
      WriteLn ('Graphics error:', GraphErrorMsg (ErrCode));
      WriteLn('Press Enter to halt()');
      ReadLn;
      Halt(100);
   end;
end;

procedure CloseGraphix;
begin
   CloseGraph;
end;

procedure Draw_Circle (X, Y : Integer; R : Integer);
begin
   Circle (X, Y, R);
end;
procedure Draw_Line (X_Start, Y_Start, X_Finish, Y_Finish : Integer);
begin
   Line (X_Start, Y_Start, X_Finish, Y_Finish)
end;

procedure Fill_Circle (X, Y : Integer; R : Integer);
begin
   FillEllipse (X, Y, R, R);
end;

procedure Set_Pixel (X, Y : Integer; Color : Color_Type);
begin
   PutPixel (X, Y, Color);
end;
function Get_Pixel (X, Y : Integer) : Color_Type;
begin
   Result := GetPixel (X, Y);
end;


procedure Set_Xor_Mode;
begin
   SetWriteMode (XorPut);
end;
procedure Set_Copy_Mode;
begin
   SetWriteMode (CopyPut);
end;

function Get_ScrWidth : Integer;
begin
   Result := GetMaxX;
end;

function Get_ScrHeight : Integer;
begin
   Result := GetMaxY;
end;

procedure Set_Color (Color : Color_Type);
begin
   SetColor (Color);
end;
procedure Set_SolidBrush (Color : Color_Type);
begin
   SetFillStyle (SolidFill, Color);
end;

end.
