{$mode objfpc}

unit u_list;

interface

type
   generic List<T> =
   class
   type public
      Item_Action = procedure (var Item : T);
      Item_Compare = function (Left, Right : T) : Boolean;

   type private
      PTItem = ^TItem;
      TItem =
      record
         info : T;
         next, prev : PTItem;
      end;

   var private
      First, Last : PTItem;

   public
      constructor Create;
      destructor Destroy; override;

      procedure Insert (Value : T);
      procedure Append (Value : T);
      procedure Clear;

      procedure For_Each (Action : Item_Action);
      function Remove_Equals (Item : T; func : Item_Compare) : Integer;

   private
      function Is_Empty : Boolean;
      function Find_Equal(Item : T; func : Item_Compare) : PTItem;
      procedure Remove_Item (p: PTItem);
   end;

implementation

constructor List.Create;
begin
   inherited;
   First := nil; Last := nil;
end;

destructor List.Destroy;
begin
   Clear;
   inherited;
end;

function List.Is_Empty : Boolean;
begin
   Result := not Assigned (First);
end;

procedure List.Insert (Value : T);
var p : PTItem;
begin
   New (p);
   p^.info := Value;
   p^.next := First;
   p^.prev := Last;

   if Is_Empty then Last := p
   else First^.prev := p;

   First := p;
end;

procedure List.Append (Value : T);
var p : PTItem;
begin
   New (p);
   p^.info := Value;
   p^.next := nil;
   p^.prev := Last;

   if Is_Empty then First := p
   else Last^.next := p;

   Last := p
end;

procedure List.Clear;
var p, pt : PTItem;
begin
   p := First;
   while assigned(p) do
   begin
      pt := p;
      p := p^.next;
      dispose(pt)
   end;
end;

procedure List.For_Each (Action : Item_Action);
var p : PTItem;
begin
   p := First;
   while Assigned(p) do
   begin
      Action(p^.info);
      p := p^.next;
   end;
end;

function List.Find_Equal (Item : T; func : Item_Compare) : PTItem;
var
   p : PTItem;
   ok : Boolean;
begin
   p := First;
   ok := True;
   while Assigned(p) and ok do
   begin
      if func(p^.info, Item) then ok := False
      else p := p^.next;
   end;
   Result := p;
end;

procedure List.Remove_Item (p: PTItem);
var r: PTItem;
begin
   r := p^.prev;
   if r <> nil then r^.next := p^.next
   else first := p^.next;

   r := p^.next;
   if r <> nil then r^.prev := p^.prev
   else last := p^.prev;

   Dispose(p);
   p := nil
end;

function List.Remove_Equals (Item : T; func : Item_Compare) : Integer;
var
   count : Integer;
   pt : PTItem;
begin
   count := 0;
   repeat
      pt := Find_Equal (Item, func);
      if Assigned(pt) then
      begin
         Remove_Item (pt);
         Inc (count);
      end;
   until (pt = nil);
   Result := count;
end;

end.
