{$mode objfpc}

unit u_scene;

interface

   // Constructor
   procedure Scene_Create;
   // Destructor
   procedure Scene_Destroy;

   // Method
   function Scene_Run : Boolean;

implementation
uses Crt, u_graph, u_equalizer, u_branch, u_snow, u_light, u_fern;

const
   Pi_Div_2 = Pi / 2.0;

const
   Snow_Amount = 400;

   procedure Generate_Snow (Count : Integer);
   var i : Integer;
   begin

      for i := 1 to Count do
      begin
         SnowList.Append (Snow_Create);
      end;

   end; { Generate_Snow }

   procedure Scene_Create;
   begin
      OpenGraphix;

      LightList := Light_List.Create;
      SnowList := Snow_List.Create;
      Branches := Branch_Vector.Create;

      Draw_Fern (1, 7 * (Get_ScrWidth div 8), Get_ScrHeight, 100.0, Pi_Div_2);
      Draw_Fern (1, 1 * (Get_ScrWidth div 8), Get_ScrHeight, 100.0, Pi_Div_2);
      Draw_Fern (1, 1 * (Get_ScrWidth div 2), Get_ScrHeight, 130.0, Pi_Div_2, True);

      LightList.For_Each (@Show_Node);

      // Show snowflakes
      Generate_Snow (Snow_Amount);

      Equalizer_Create (Ground);
   end; { Scene_Create }

   procedure Scene_Destroy;
   begin
      Branches.Destroy;
      SnowList.Destroy;
      LightList.Destroy;

      CloseGraphix;
   end; { Scene_Destroy }

   function Scene_Run : Boolean;
   const
      To_Delay = 55;
   var
      Count : Integer;
   begin
      Result := True;

      SnowList.For_Each (@Acc_Refresh);
      SnowList.Remove_Equals (Snow_Compare (False, True),
                              @Snow_Equals);

      Count := SnowList.Remove_Equals (Snow_Compare (False, False),
                                       @Snow_Equals);

      Generate_Snow (Count);

      LightList.For_Each (@Light_Lamp);

      Delay(To_Delay);
      if KeyPressed then
      begin
         case ReadKey of
            #27: Result := False;
         end;
      end;

   end; { Scene_Run }

end.
