{$mode objfpc}
unit u_snow;

interface
uses u_list;

type
   TSnow =
   record
      X, Y : Integer;
      Vx, Vy : Double;

      Active : Boolean;

      // Private
      Mass : Integer;
      Has_Under : Boolean;
      Under : Integer;
      Fall : Boolean;
   end;

var
   Snow_Count : Integer = 0;
   Fall_Count : Integer = 0;

   // Constructors
   function Snow_Compare (TheActive, TheFall : Boolean) : TSnow;
   function Snow_Create : TSnow;
   function Snow_Falling (Px, Py, AMass : Integer) : TSnow;

   // Methods
   procedure Show (var S : TSnow);
   procedure Hide (var S : TSnow);
   procedure Recalc (var S : TSnow);

type
   Snow_List = specialize List<TSnow>;
var
   SnowList : Snow_List;


   procedure Acc_Refresh (var S : TSnow);
   function Snow_Equals (Left, Right : TSnow) : Boolean;

implementation
uses u_graph, u_branch, u_equalizer;

const
   // Snow generator Constants
   Wind_Probability = 100; // of 10_000
   VScale = 1.25;

   // Operator overloading
   operator = (Left, Right : TSnow) R : Boolean;
   begin
      R := (Left.Active = Right.Active) and (Left.Fall = Right.Fall);
   end;

   function Snow_Equals (Left, Right : TSnow) : Boolean;
   begin
      Result := (Left = Right);
   end;

   // Access (Iterate) functions
   procedure Acc_Append_Fall (Count : Integer; Left, Height, Width, Speed : Integer);
   var i : Integer;
   begin
      for i := 1 to Count do
      begin
         SnowList.Append (Snow_Falling (Left + Random(Width), Height, Speed + (Random (5))));
      end;
   end; { Acc_Append_Fall }

   procedure Acc_Refresh (var S : TSnow);
   begin
      if S.Active then
      begin
         Hide (S);
         Recalc (S);
         Show (S);
      end;
   end; { Acc_Refresh }


   // Methods
   procedure Hide (var S : TSnow);
   begin
      if S.Has_Under then
      begin
         S.Has_Under := False;
         Set_Pixel (S.X, S.Y, S.Under);
      end;
   end; { Hide }

   procedure Recalc (var S : TSnow);
   var
      range_out : Boolean;
      stop_calc : Boolean;

      br : TBranch;
      value : Integer;
      i : Integer;

   const
      Fall_Max = 10;
   begin
      if Random (10000) < Wind_Probability then
      begin
         S.Vx := -S.Vx;
      end;
      S.X := Trunc (S.X + S.Vx);
      S.Y := Trunc (S.Y + S.Vy);

      stop_calc := false;
      for i := 0 to Branches.Count - 1 do
      begin
         br := Branches.Element (i);
         if
           (S.X >= Get_Left (br)) and
           (S.X < Get_Right (br)) and
           (S.Y <= Get_Height (br)) and
           (S.Y >= Get_Height (br) - Get_Arr (br, S.X - Get_Left (br)))
         then
         begin
            u_branch.Update (br, S.X, S.Mass, @Acc_Append_Fall);
            Hide (S);
            S.Active := false;
            stop_calc := true;
            Break;
         end;
      end;

      if not stop_calc then
      begin
         value := Get_Value (Ground, S.X, range_out);
         if S.Y >= value then
         begin
            if not range_out then
            begin
               if not S.Fall then
                  Update (Ground, S.X)
               else
               begin
                  if Fall_Count = 0 then
                  begin
                     Update (Ground, S.X);
                     Fall_Count := Fall_Count + 1;
                  end
                  else
                  begin
                     Fall_Count := Fall_Count + 1;
                     if Fall_Count > Fall_Max then
                     begin
                        Fall_Count := 0;
                     end;
                  end;
               end;
            end;
            S.Active := false;
         end;
      end;
   end; { Recalc }

   procedure Show (var S : TSnow);
   begin
      S.Under := Get_Pixel (S.X, S.Y);
      if S.Under = White_Color then
         S.Has_Under := false
      else
      begin
         S.Has_Under := true;
         Set_Pixel (S.X, S.Y, White_Color);
      end;
   end; { Show }

   // Constructors
   function Snow_Compare (TheActive, TheFall : Boolean) : TSnow;
   begin
      with Result do
      begin
         Fall := TheFall;
         Mass := 0;
         X := 0;
         Y := 0;
         Vx := 0.0;
         Vy := 0.0;
         Active := TheActive;
         Has_Under := False;
         Under := Black_Color;
      end;
   end; { Snow_Compare }

   function Snow_Create : TSnow;
   const
      Every = 2000;
   var
      The_Mass : Integer;
   begin
      if Snow_Count = Every then
      begin
         Snow_Count := 0;
         Decrease_Level (Ground);
      end;

      The_Mass := Random (5) + 1;
      with Result do
      begin
         Fall := False;
         Mass := The_Mass;
         X := Random (Get_ScrWidth);
         Y := Random (Get_ScrHeight div 4);
         Vx := Random (5) - 2.0;
         Vy := The_Mass * VScale;
         Active := True;
         Has_Under := False;
         Under := White_Color;
      end;
   end; { Snow_Create; }

   function Snow_Falling (Px, Py, AMass : Integer) : TSnow;
   begin
      with Result do
      begin
         Fall := True;
         Mass := AMass;
         X := Px;
         Y := Py;
         Vx := 0.0;
         Vy := AMass * VScale;
         Active := True;
         Has_Under := False;
         Under := White_Color;
      end;
   end; { Snow_Falling; }

end.
